@@ -1,7 +1,7 @@
 package App::BencherUtils;
 
-our $DATE = '2016-01-08'; # DATE
-our $VERSION = '0.03'; # VERSION
+our $DATE = '2016-01-11'; # DATE
+our $VERSION = '0.04'; # VERSION
 
 use 5.010001;
 use strict;
@@ -145,21 +145,29 @@ sub bencher_all {
 
   FILTER:
     {
+        # normalize
+        my $includes = [
+            map {s!/!::!g; $_} @{ $args{includes} // [] }
+        ];
+        my $excludes = [
+            map {s!/!::!g; $_} @{ $args{excludes} // [] }
+        ];
+
         my $filtered;
-        if ($args{includes} && @{ $args{includes} }) {
+        if (@$includes) {
             $filtered++;
             my @fscenarios;
             for my $s (@scenarios) {
-                next unless grep {$s eq $_} @{ $args{includes} };
+                next unless grep {$s eq $_} @$includes;
                 push @fscenarios, $s;
             }
             @scenarios = @fscenarios;
         }
-        if ($args{excludes} && @{ $args{excludes} }) {
+        if (@$excludes) {
             $filtered++;
             my @fscenarios;
             for my $s (@scenarios) {
-                next if grep {$s eq $_} @{ $args{excludes} };
+                next if grep {$s eq $_} @$excludes;
                 push @fscenarios, $s;
             }
             @scenarios = @fscenarios;
@@ -325,6 +333,12 @@ $SPEC{list_bencher_results} = {
             schema => 'bool*',
             tags => ['category:filtering'],
         },
+        latest => {
+            'summary.alt.bool.yes' => 'Only list the latest result for every scenario+CPU',
+            'summary.alt.bool.no'  => 'Do not list the latest result for every scenario+CPU',
+            schema => ['bool*'],
+            tags => ['category:filtering'],
+        },
 
         fmt => {
             summary => 'Display each result with bencher-fmt',
@@ -372,6 +386,15 @@ sub list_bencher_results {
 
     opendir my($dh), $dir or return [500, "Can't read results_dir `$dir`: $!"];
 
+    # normalize
+    my $include_scenarios = [
+        map {s!/!::!g; $_} @{ $args{include_scenarios} // [] }
+    ];
+    my $exclude_scenarios = [
+        map {s!/!::!g; $_} @{ $args{exclude_scenarios} // [] }
+    ];
+
+    my %latest; # key = module+(module_startup)+cpu, value = latest row
     my @rows;
   FILE:
     for my $filename (sort readdir $dh) {
@@ -383,11 +406,11 @@ sub list_bencher_results {
             time => "$3-$4-$5T$6:$7:$8",
         };
         $row->{scenario} =~ s/-/::/g;
-        if ($args{include_scenarios} && @{ $args{include_scenarios} }) {
-            next FILE unless grep {$row->{scenario} eq $_} @{ $args{include_scenarios} };
+        if (@$include_scenarios) {
+            next FILE unless grep {$row->{scenario} eq $_} @$include_scenarios;
         }
-        if ($args{exclude_scenarios} && @{ $args{exclude_scenarios} }) {
-            next FILE if grep {$row->{scenario} eq $_} @{ $args{exclude_scenarios} };
+        if (@$exclude_scenarios) {
+            next FILE if grep {$row->{scenario} eq $_} @$exclude_scenarios;
         }
 
         my $benchres = _json->decode(File::Slurper::read_text("$dir/$filename"));
@@ -400,6 +423,13 @@ sub list_bencher_results {
             next FILE if $row->{module_startup} xor $args{module_startup};
         }
 
+        my $key = sprintf(
+            "%s.%s.%s",
+            $row->{scenario},
+            $row->{module_startup} ? 0:1,
+            $row->{cpu},
+        );
+
         if ($args{query} && @{ $args{query} }) {
             my $matches = 1;
           QUERY_WORD:
@@ -416,9 +446,27 @@ sub list_bencher_results {
             next unless $matches;
         }
 
+        if (!$latest{$key} || $latest{$key}{time} lt $row->{time}) {
+            $latest{$key} = $row;
+        }
+        $row->{_key} = $key;
+
         push @rows, $row;
     }
 
+    # we do the 'latest' filter here after we get all the rows
+    if (defined $args{latest}) {
+        my @rows0 = @rows;
+        @rows = grep {
+            my $latest_time = $latest{ $_->{_key} }{time};
+            $args{latest} ?
+                $_->{time} eq $latest_time :
+                $_->{time} ne $latest_time;
+        } @rows;
+    }
+
+    for (@rows) { delete $_->{_key} }
+
     my $resmeta = {};
     if ($args{fmt}) {
         require Bencher;
@@ -460,7 +508,7 @@ App::BencherUtils - Utilities related to bencher
 
 =head1 VERSION
 
-This document describes version 0.03 of App::BencherUtils (from Perl distribution App-BencherUtils), released on 2016-01-08.
+This document describes version 0.04 of App::BencherUtils (from Perl distribution App-BencherUtils), released on 2016-01-11.
 
 =head1 SYNOPSIS
 
@@ -640,6 +688,8 @@ Display each result with bencher-fmt.
 
 =item * B<include_scenarios> => I<array[str]>
 
+=item * B<latest> => I<bool>
+
 =item * B<module_startup> => I<bool>
 
 =item * B<query> => I<array[str]>
